"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
var Rx_1 = require('rxjs/Rx');
var config_1 = require('../config');
require('rxjs/add/operator/map');
var ApiAuthorisationError = (function (_super) {
    __extends(ApiAuthorisationError, _super);
    function ApiAuthorisationError(options) {
        _super.call(this, 'API Authorisation error');
        Object.assign(this, options);
    }
    return ApiAuthorisationError;
}(Error));
var ApiService = (function () {
    function ApiService(http, config, apiTokenStore, authTokensValidator) {
        this.http = http;
        this.config = config;
        this.apiTokenStore = apiTokenStore;
        this.authTokensValidator = authTokensValidator;
    }
    ApiService.prototype.fetch = function (path, options) {
        var url = this.getPath(path);
        return this.http.get(url, options).map(this.handleResponse);
    };
    ApiService.prototype.authorisedFetch = function (path, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        var tokens = this.apiTokenStore.get();
        if (!tokens) {
            this.apiTokenStore.events.error(new ApiAuthorisationError({
                missingTokens: true
            }));
            return Rx_1.Observable.empty();
        }
        return this.authTokensValidator
            .validateTokens(tokens.authToken, tokens.refreshToken)
            .flatMap(function (validatedTokens) {
            options.headers = options.headers || new http_1.Headers();
            options.headers.set('Authorization', "bearer " + validatedTokens.authToken);
            if (validatedTokens.hasBeenRefreshed) {
                _this.apiTokenStore.events.next(validatedTokens);
            }
            return _this.fetch(path, options);
        })
            .catch(function (err) {
            _this.apiTokenStore.events.error(new ApiAuthorisationError({
                invalidAuthToken: true
            }));
            return Rx_1.Observable.empty();
        });
    };
    ApiService.prototype.post = function (path, data, options) {
        var url = this.getPath(path);
        return this.http.post(url, JSON.stringify(data), options)
            .map(this.handleResponse);
    };
    ApiService.prototype.getPath = function (path) {
        return this.config.baseUrl + "/" + path;
    };
    ApiService.prototype.handleResponse = function (res) {
        if (res.status >= 400) {
            throw new ApiError('Bad status code', res);
        }
        return res.json();
    };
    ApiService = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [http_1.Http, config_1.Config, Object, Object])
    ], ApiService);
    return ApiService;
}());
exports.ApiService = ApiService;
var ApiError = (function (_super) {
    __extends(ApiError, _super);
    function ApiError(message, response) {
        _super.call(this, message);
        this.response = response;
    }
    ;
    return ApiError;
}(Error));
exports.ApiError = ApiError;
//# sourceMappingURL=api.js.map