"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var http_1 = require('@angular/http');
var testing_1 = require('@angular/http/testing');
var testing_2 = require('@angular/core/testing');
var Rx_1 = require('rxjs/Rx');
var api_1 = require('./api');
var config_1 = require('../config');
var Rx_2 = require('rxjs/Rx');
var TestConfig = (function (_super) {
    __extends(TestConfig, _super);
    function TestConfig() {
        _super.apply(this, arguments);
        this.baseUrl = 'http://www.example.com';
    }
    return TestConfig;
}(config_1.Config));
testing_2.describe('ApiService', function () {
    var apiTokens, apiTokenStore, fakeAuthTokensValidator, fakeAuthTokensValidatorResult;
    var FakeApiTokenStore = (function () {
        function FakeApiTokenStore() {
            this.events = new Rx_1.Subject();
        }
        FakeApiTokenStore.prototype.get = function () {
            return apiTokens;
        };
        return FakeApiTokenStore;
    }());
    beforeEach(function () {
        fakeAuthTokensValidatorResult = Rx_2.Observable.from([{
                authToken: 'validatedAuthToken',
                refreshToken: 'validatedRefreshToken',
                hasBeenRefreshed: false
            }]);
        apiTokenStore = new FakeApiTokenStore();
        fakeAuthTokensValidator = {
            validateTokens: function (authToken, refreshToken) {
                return fakeAuthTokensValidatorResult;
            }
        };
        apiTokens = {
            authToken: 'authToken',
            refreshToken: 'refreshToken'
        };
    });
    testing_2.beforeEachProviders(function () { return [
        {
            provide: api_1.ApiService,
            deps: [http_1.Http, config_1.Config],
            useFactory: function (http, config) {
                return new api_1.ApiService(http, config, apiTokenStore, fakeAuthTokensValidator);
            }
        },
        testing_1.MockBackend,
        http_1.BaseRequestOptions,
        {
            provide: http_1.Http,
            useFactory: function (backend, options) { return new http_1.Http(backend, options); },
            deps: [testing_1.MockBackend, http_1.BaseRequestOptions]
        },
        {
            provide: config_1.Config,
            useClass: TestConfig
        }
    ]; });
    function makeResponse(response) {
        return new http_1.Response(new http_1.ResponseOptions(response));
    }
    function testApiCall(options) {
        return testing_2.async(testing_2.inject([api_1.ApiService, testing_1.MockBackend], function (api, mockBackEnd) {
            var response = { body: options.stubbedBody, status: options.stubbedStatusCode || 200 };
            mockBackEnd.connections.subscribe(function (connection) {
                testing_2.expect(connection.request.url)
                    .toEqual("http://www.example.com/" + options.expectedPath);
                if (options.expectedBody) {
                    testing_2.expect(JSON.parse(connection.request.getBody()))
                        .toEqual(options.expectedBody);
                }
                if (options.expectedHeaders) {
                    Object.keys(options.expectedHeaders).forEach(function (header) {
                        testing_2.expect(connection.request.headers.getAll(header))
                            .toEqual(options.expectedHeaders[header]);
                    });
                }
                if (options.stubbedError) {
                    return connection.mockError(options.stubbedError);
                }
                connection.mockRespond(makeResponse(response));
            });
            return options.test(api);
        }));
    }
    testing_2.it('returns the json when response is 200 for post requests', testApiCall({
        expectedPath: 'lists/666',
        expectedHeaders: { someHeader: ['blah'] },
        expectedBody: { bodyData: 'fleh' },
        stubbedBody: { id: 123 },
        test: function (api) {
            var options = {
                headers: new http_1.Headers({ someHeader: 'blah' })
            };
            api.post('lists/666', { bodyData: 'fleh' }, options)
                .subscribe(function (data) { return testing_2.expect(data).toEqual({ id: 123 }); });
        }
    }));
    testing_2.it('returns the json when response is 200 for fetch requests', testApiCall({
        expectedPath: 'lists/666',
        expectedHeaders: { someHeader: ['woo'] },
        stubbedBody: { id: 123 },
        test: function (api) {
            var options = {
                headers: new http_1.Headers({ someHeader: 'woo' })
            };
            api.fetch('lists/666', options)
                .subscribe(function (data) { return testing_2.expect(data).toEqual({ id: 123 }); });
        }
    }));
    testing_2.it('returns the json when response is 200 for authorised fetch requests', testApiCall({
        expectedPath: 'lists/666',
        expectedHeaders: { someHeader: ['wee'] },
        stubbedBody: { id: 123 },
        test: function (api) {
            var options = {
                headers: new http_1.Headers({ someHeader: 'wee' })
            };
            api.authorisedFetch('lists/666', options)
                .subscribe(function (data) { return testing_2.expect(data).toEqual({ id: 123 }); });
        }
    }));
    testing_2.it('appends the authorization header to authorised fetch requests', testApiCall({
        expectedPath: 'test',
        expectedHeaders: { Authorization: ['bearer validatedAuthToken'] },
        test: function (api) { return api.authorisedFetch('test').subscribe(); }
    }));
    testing_2.describe('with an empty api token store', function () {
        beforeEach(function () { return apiTokens = undefined; });
        testing_2.it('raises unauthorised event when an authorised fetch request is made', testApiCall({
            expectedPath: 'test',
            test: function (api) {
                apiTokenStore.events.subscribe(function () { return fail(new Error('Expected error event')); }, function (err) { return testing_2.expect(err.missingTokens).toEqual(true); });
                api.authorisedFetch('test').subscribe();
            }
        }));
    });
    testing_2.describe('when api token validation fails', function () {
        beforeEach(function () {
            fakeAuthTokensValidatorResult = Rx_2.Observable.throw(new Error('plop'));
        });
        testing_2.it('raises unauthorised event when an authorised fetch request is made', testApiCall({
            expectedPath: 'test',
            test: function (api) {
                apiTokenStore.events.subscribe(function () { return fail(new Error('Expected error event')); }, function (err) { return testing_2.expect(err.invalidAuthToken).toEqual(true); });
                api.authorisedFetch('test').subscribe();
            }
        }));
    });
    testing_2.describe('when the api token has been refreshed', function () {
        beforeEach(function () {
            fakeAuthTokensValidatorResult = Rx_2.Observable.from([{
                    authToken: 'new-token',
                    refreshToken: 'new-refresh-token',
                    hasBeenRefreshed: true
                }]);
        });
        testing_2.it('emits a refresh event', testApiCall({
            expectedPath: 'test',
            test: function (api) {
                apiTokenStore.events.subscribe(function (evt) { return testing_2.expect(evt).toEqual({
                    hasBeenRefreshed: true,
                    authToken: 'new-token',
                    refreshToken: 'new-refresh-token'
                }); }, function (err) { return fail(err); });
                api.authorisedFetch('test').subscribe();
            }
        }));
    });
    testing_2.describe('when the api token has not been refreshed', function () {
        beforeEach(function () {
            fakeAuthTokensValidatorResult = Rx_2.Observable.from([{
                    authToken: 'old-token',
                    refreshToken: 'old-refresh-token',
                    hasBeenRefreshed: false
                }]);
        });
        testing_2.it('does not emit a refresh event', testApiCall({
            expectedPath: 'test',
            test: function (api) {
                apiTokenStore.events.subscribe(function (evt) { return fail(new Error('should not emit an event')); }, function (err) { return fail(err); });
                api.authorisedFetch('test').subscribe();
            }
        }));
    });
    testing_2.it('throws an error when the post response body is not valid json', testApiCall({
        expectedPath: 'blah',
        stubbedBody: 'not json',
        test: function (api) { return api.post('blah', 'request body')
            .subscribe(function (data) { }, function (err) { return testing_2.expect(err).toEqual(jasmine.any(Error)); }); }
    }));
    testing_2.it('throws an error when the fetch response body is not valid json', testApiCall({
        expectedPath: 'blah',
        stubbedBody: 'not json',
        test: function (api) { return api.fetch('blah')
            .subscribe(function (data) { }, function (err) { return testing_2.expect(err).toEqual(jasmine.any(Error)); }); }
    }));
    testing_2.it('throws an error when the authorised fetch response body is not valid json', testApiCall({
        expectedPath: 'blah',
        stubbedBody: 'not json',
        test: function (api) { return api.authorisedFetch('blah')
            .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) { return testing_2.expect(err).toEqual(jasmine.any(Error)); }); }
    }));
    var errorStatusCodes = [404, 500];
    errorStatusCodes.forEach(function (statusCode) {
        testing_2.it("throws an error when post response is " + statusCode, testApiCall({
            expectedPath: 'blah',
            stubbedStatusCode: statusCode,
            test: function (api) { return api.post('blah', {})
                .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) {
                testing_2.expect(err).toEqual(jasmine.any(Error));
                testing_2.expect(err.response.status).toEqual(statusCode);
            }); }
        }));
        testing_2.it("throws an error when fetch response is " + statusCode, testApiCall({
            expectedPath: 'blah',
            stubbedStatusCode: statusCode,
            test: function (api) { return api.fetch('blah')
                .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) {
                testing_2.expect(err).toEqual(jasmine.any(Error));
                testing_2.expect(err.response.status).toEqual(statusCode);
            }); }
        }));
        testing_2.it("throws an error when authorised fetch response is " + statusCode, testApiCall({
            expectedPath: 'blah',
            stubbedStatusCode: statusCode,
            test: function (api) { return api.authorisedFetch('blah')
                .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) {
                testing_2.expect(err).toEqual(jasmine.any(Error));
                testing_2.expect(err.response.status).toEqual(statusCode);
            }); }
        }));
    });
    testing_2.it('fails post requests gracefully when the API is unavailable', testApiCall({
        expectedPath: 'blah',
        stubbedError: new Error('boom'),
        test: function (api) { return api.post('blah', {})
            .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) { return testing_2.expect(err).toEqual(jasmine.any(Error)); }); }
    }));
    testing_2.it('fails fetch requests gracefully when the API is unavailable', testApiCall({
        expectedPath: 'blah',
        stubbedError: new Error('boom'),
        test: function (api) { return api.fetch('blah')
            .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) { return testing_2.expect(err).toEqual(jasmine.any(Error)); }); }
    }));
    testing_2.it('fails authorised fetch requests gracefully when the API is unavailable', testApiCall({
        expectedPath: 'blah',
        stubbedError: new Error('boom'),
        test: function (api) { return api.authorisedFetch('blah')
            .subscribe(function () { return fail(new Error('ApiService should error')); }, function (err) { return testing_2.expect(err).toEqual(jasmine.any(Error)); }); }
    }));
});
//# sourceMappingURL=api.test.js.map